"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.parseComment = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _commentParser = _interopRequireDefault(require("comment-parser"));

var _jsdocUtils = _interopRequireDefault(require("./jsdocUtils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const parseComment = (commentNode, indent) => {
  // Preserve JSDoc block start/end indentation.
  return (0, _commentParser.default)(indent + '/*' + commentNode.value + indent + '*/', {
    // @see https://github.com/yavorskiy/comment-parser/issues/21
    parsers: [_commentParser.default.PARSERS.parse_tag, _commentParser.default.PARSERS.parse_type, (str, data) => {
      if (_lodash.default.includes(['return', 'returns', 'throws', 'exception'], data.tag)) {
        return null;
      }

      return _commentParser.default.PARSERS.parse_name(str, data);
    }, _commentParser.default.PARSERS.parse_description]
  })[0] || {};
};

exports.parseComment = parseComment;

const curryUtils = (functionNode, jsdoc, tagNamePreference, exampleCodeRegex, rejectExampleCodeRegex, additionalTagNames, baseConfig, configFile, captionRequired, matchingFileName, eslintrcForExamples, allowInlineConfig, allowEmptyNamepaths, reportUnusedDisableDirectives, noDefaultExampleRules, allowOverrideWithoutParam, allowImplementsWithoutParam, allowAugmentsExtendsWithoutParam, checkSeesForNamepaths, ancestors, sourceCode) => {
  const utils = {};

  utils.getFunctionParameterNames = () => {
    return _jsdocUtils.default.getFunctionParameterNames(functionNode);
  };

  utils.getFunctionSourceCode = () => {
    return sourceCode.getText(functionNode);
  };

  utils.isConstructor = () => {
    return functionNode.parent && functionNode.parent.kind === 'constructor';
  };

  utils.getJsdocParameterNamesDeep = () => {
    return _jsdocUtils.default.getJsdocParameterNamesDeep(jsdoc, utils.getPreferredTagName('param'));
  };

  utils.getJsdocParameterNames = () => {
    return _jsdocUtils.default.getJsdocParameterNames(jsdoc, utils.getPreferredTagName('param'));
  };

  utils.getPreferredTagName = name => {
    return _jsdocUtils.default.getPreferredTagName(name, tagNamePreference);
  };

  utils.getExampleCodeRegex = () => {
    return exampleCodeRegex;
  };

  utils.getRejectExampleCodeRegex = () => {
    return rejectExampleCodeRegex;
  };

  utils.getMatchingFileName = () => {
    return matchingFileName;
  };

  utils.isValidTag = name => {
    return _jsdocUtils.default.isValidTag(name, additionalTagNames);
  };

  utils.hasATag = name => {
    return _jsdocUtils.default.hasATag(jsdoc, name);
  };

  utils.hasTag = name => {
    return _jsdocUtils.default.hasTag(jsdoc, name);
  };

  utils.useEslintrcForExamples = () => {
    return eslintrcForExamples;
  };

  utils.allowInlineConfig = () => {
    return allowInlineConfig;
  };

  utils.reportUnusedDisableDirectives = () => {
    return reportUnusedDisableDirectives;
  };

  utils.hasNoDefaultExampleRules = () => {
    return noDefaultExampleRules;
  };

  utils.getBaseConfig = () => {
    return baseConfig;
  };

  utils.getConfigFile = () => {
    return configFile;
  };

  utils.isCaptionRequired = () => {
    return captionRequired;
  };

  utils.isOverrideAllowedWithoutParam = () => {
    return allowOverrideWithoutParam;
  };

  utils.isImplementsAllowedWithoutParam = () => {
    return allowImplementsWithoutParam;
  };

  utils.isAugmentsExtendsAllowedWithoutParam = () => {
    return allowAugmentsExtendsWithoutParam;
  };

  utils.isNamepathType = tagName => {
    return _jsdocUtils.default.isNamepathType(tagName, checkSeesForNamepaths);
  };

  utils.passesEmptyNamepathCheck = tag => {
    return !tag.name && allowEmptyNamepaths && _lodash.default.includes([// These may serve some minor purpose when empty
    'callback', 'event', 'listens', 'fires', 'emits'], tag.tag);
  };

  utils.getClassJsdocNode = () => {
    const greatGrandParent = ancestors.slice(-3)[0];
    const greatGrandParentValue = greatGrandParent && sourceCode.getFirstToken(greatGrandParent).value;

    if (greatGrandParentValue === 'class') {
      const classJsdocNode = sourceCode.getJSDocComment(greatGrandParent);
      return classJsdocNode;
    }

    return false;
  };

  utils.classHasTag = tagName => {
    const classJsdocNode = utils.getClassJsdocNode();

    if (classJsdocNode) {
      const indent = _lodash.default.repeat(' ', classJsdocNode.loc.start.column);

      const classJsdoc = parseComment(classJsdocNode, indent);

      if (_jsdocUtils.default.hasTag(classJsdoc, tagName)) {
        return true;
      }
    }

    return false;
  };

  return utils;
};

var _default = iterator => {
  return context => {
    const sourceCode = context.getSourceCode();
    const tagNamePreference = _lodash.default.get(context, 'settings.jsdoc.tagNamePreference') || {};
    const exampleCodeRegex = _lodash.default.get(context, 'settings.jsdoc.exampleCodeRegex') || null;
    const rejectExampleCodeRegex = _lodash.default.get(context, 'settings.jsdoc.rejectExampleCodeRegex') || null;
    const matchingFileName = _lodash.default.get(context, 'settings.jsdoc.matchingFileName') || null;
    const additionalTagNames = _lodash.default.get(context, 'settings.jsdoc.additionalTagNames') || {};
    const baseConfig = _lodash.default.get(context, 'settings.jsdoc.baseConfig') || {};

    const configFile = _lodash.default.get(context, 'settings.jsdoc.configFile');

    const eslintrcForExamples = _lodash.default.get(context, 'settings.jsdoc.eslintrcForExamples') !== false;
    const allowInlineConfig = _lodash.default.get(context, 'settings.jsdoc.allowInlineConfig') !== false;
    const allowEmptyNamepaths = _lodash.default.get(context, 'settings.jsdoc.allowEmptyNamepaths') !== false;
    const reportUnusedDisableDirectives = _lodash.default.get(context, 'settings.jsdoc.reportUnusedDisableDirectives') !== false;
    const captionRequired = Boolean(_lodash.default.get(context, 'settings.jsdoc.captionRequired'));
    const noDefaultExampleRules = Boolean(_lodash.default.get(context, 'settings.jsdoc.noDefaultExampleRules'));
    const allowOverrideWithoutParam = Boolean(_lodash.default.get(context, 'settings.jsdoc.allowOverrideWithoutParam'));
    const allowImplementsWithoutParam = Boolean(_lodash.default.get(context, 'settings.jsdoc.allowImplementsWithoutParam'));
    const allowAugmentsExtendsWithoutParam = Boolean(_lodash.default.get(context, 'settings.jsdoc.allowAugmentsExtendsWithoutParam'));
    const checkSeesForNamepaths = Boolean(_lodash.default.get(context, 'settings.jsdoc.checkSeesForNamepaths'));

    const checkJsdoc = functionNode => {
      const jsdocNode = sourceCode.getJSDocComment(functionNode);

      if (!jsdocNode) {
        return;
      }

      const ancestors = context.getAncestors();

      const indent = _lodash.default.repeat(' ', jsdocNode.loc.start.column);

      const jsdoc = parseComment(jsdocNode, indent);

      const report = (message, fixer = null, jsdocLoc = null) => {
        let loc;

        if (jsdocLoc) {
          const lineNumber = jsdocNode.loc.start.line + jsdocLoc.line;
          loc = {
            end: {
              line: lineNumber
            },
            start: {
              line: lineNumber
            }
          };

          if (jsdocLoc.column) {
            const colNumber = jsdocNode.loc.start.column + jsdocLoc.column;
            loc.end.column = colNumber;
            loc.start.column = colNumber;
          }
        }

        if (fixer === null) {
          context.report({
            loc,
            message,
            node: jsdocNode
          });
        } else {
          context.report({
            fix: fixer,
            loc,
            message,
            node: jsdocNode
          });
        }
      };

      const utils = curryUtils(functionNode, jsdoc, tagNamePreference, exampleCodeRegex, rejectExampleCodeRegex, additionalTagNames, baseConfig, configFile, captionRequired, matchingFileName, eslintrcForExamples, allowInlineConfig, allowEmptyNamepaths, reportUnusedDisableDirectives, noDefaultExampleRules, allowOverrideWithoutParam, allowImplementsWithoutParam, allowAugmentsExtendsWithoutParam, checkSeesForNamepaths, ancestors, sourceCode);
      iterator({
        context,
        functionNode,
        indent,
        jsdoc,
        jsdocNode,
        report,
        sourceCode,
        utils
      });
    };

    return {
      ArrowFunctionExpression: checkJsdoc,
      FunctionDeclaration: checkJsdoc,
      FunctionExpression: checkJsdoc
    };
  };
};

exports.default = _default;
//# sourceMappingURL=iterateJsdoc.js.map